from fabric.api import *
import os.path as path
import sys, time, os
import shutil


"""
---------------------------------------------------------------
CHANGE THESE VERSION NUMBERS TO DEPLOY A NEW VERSION TO UNICORN
---------------------------------------------------------------
"""

DEV_ANALYTICS_ROLLBACK_VERSION = '3.92'
DEV_ANALYTICS_VERSION = '3.97'

PROD_ANALYTICS_ROLLBACK_VERSION = '3.91'
PROD_ANALYTICS_VERSION = '3.92'  # Beware, changing this will update the PROD version!

"""
---------------------------------------------------------------
"""

DEPLOYMENT_PREFIX = '/data/sys_analytics'
ARTIFACT_FILE_PATH = '/sw/analytics/analytics-server-%s.jar'
ANALYTICS_SERVER_RUN_SCRIPT_PATH = '/service/sys_analytics/run'


def deploy(prod=False, rollback=False):
    if rollback:
        # Work out the right rollback deploy version
        deploy_version = DEV_ANALYTICS_ROLLBACK_VERSION if not prod else PROD_ANALYTICS_ROLLBACK_VERSION
        # Copy the old run script back
        shutil.copy('./run.old', ANALYTICS_SERVER_RUN_SCRIPT_PATH)
    else:
        # Work out the right deploy version
        deploy_version = DEV_ANALYTICS_VERSION if not prod else PROD_ANALYTICS_VERSION
        # Copy the new run script, keeping the old one for rollback
        shutil.copy(ANALYTICS_SERVER_RUN_SCRIPT_PATH, './run.old')
        shutil.copy('./run', ANALYTICS_SERVER_RUN_SCRIPT_PATH)
        os.chmod(ANALYTICS_SERVER_RUN_SCRIPT_PATH, 0755)

    # setup some paths
    current = path.join(DEPLOYMENT_PREFIX, 'bin/analytics-server.jar')
    new = ARTIFACT_FILE_PATH % deploy_version
    puts('artifact path: %s' % new)

    if not path.exists(new):
        puts('version artifact could not be found, make sure it has been promoted by SWT properly')
        return

    if path.exists(current):
        # check which version is currently deployed
        readlink = os.readlink(current)
        if readlink == new:
            puts('%s is already deployed' % deploy_version)
            return
        else:
            # deploying a different version to what is currently there
            puts('currently deployed is %s, changing to deploy %s' % (readlink, deploy_version))
    else:
        # usually the initial deploy, the symlink is missing
        puts('could not read current version, deploying %s' % deploy_version)

    # link current -> version we just uploaded
    local('ln -sfn "%s" "%s"' % (new, current))


def restart_server():
    local('svc -t /service/sys_analytics')


def check_server_alive():
    """Ping the healthcheck endpoint to ensure the server is alive"""

    hostname = local('hostname', capture=True)

    retry = 20
    while retry:
        # with hide('output'):
        status = local('curl -sL -w "%%{http_code}" "http://%s:8080/analytics/healthcheck" -o /dev/null' % hostname, capture=True)
        if status != '200':
            puts('Couldn\'t connect to healthcheck, retry in 5s. Status code returned: %s' % status)
        else:
            puts('Healthcheck returned 200 OK')
            return True

        time.sleep(5)
        retry -= 1

    # too many retries
    puts('server failed to respond to healthcheck in a timeline manner')
    return False


if __name__ == '__main__':
    prod_override = False
    if len(sys.argv) > 0 and sys.argv[0].lower() == "prod":
        puts('overriding deployment environment to prod')
        prod_override = True

    deploy(prod=prod_override)
    restart_server()
    if not check_server_alive():
        deploy(prod=prod_override, rollback=True)
        restart_server()
        if not check_server_alive():
            abort('could not deploy new version OR roll back - this is bad, fix it manually!')
        sys.exit(1)
